@@ -40,6 +40,7 @@ GENERATED_PROVIDERS_DEPENDENCIES_FILE = AIRFLOW_ROOT_PATH / "generated" / "provi
 PROVIDER_DEPENDENCIES = json.loads(GENERATED_PROVIDERS_DEPENDENCIES_FILE.read_text())
 
 PRE_INSTALLED_PROVIDERS = [
+    "common.compat",
     "common.io",
     "common.sql",
     "fab>=1.0.2",
@@ -58,7 +59,7 @@ CORE_EXTRAS: dict[str, list[str]] = {
     # TODO: We can remove it once boto3 and aiobotocore both have compatible botocore version or
     # boto3 have native aync support and we move away from aio aiobotocore
     "aiobotocore": [
-        "aiobotocore>=2.7.0",
+        "aiobotocore>=2.9.0",
     ],
     "async": [
         "eventlet>=0.33.3",
@@ -75,6 +76,9 @@ CORE_EXTRAS: dict[str, list[str]] = {
         # Cgroupspy 0.2.2 added Python 3.10 compatibility
         "cgroupspy>=0.2.2",
     ],
+    "cloudpickle": [
+        "cloudpickle",
+    ],
     "deprecated-api": [
         "requests>=2.27.0,<3",
     ],
@@ -197,6 +201,8 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
         "click>=8.0",
         "gitpython>=3.1.40",
         "hatch>=1.9.1",
+        # Incremental 24.7.0, 24.7.1 has broken `python -m virtualenv` command when run in /opt/airflow directory
+        "incremental!=24.7.0,!=24.7.1,>=22.10.0",
         "pipdeptree>=2.13.1",
         "pygithub>=2.1.1",
         "restructuredtext-lint>=1.4.0",
@@ -245,7 +251,7 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
     "devel-static-checks": [
         "black>=23.12.0",
         "pre-commit>=3.5.0",
-        "ruff==0.3.3",
+        "ruff==0.5.5",
         "yamllint>=1.33.0",
     ],
     "devel-tests": [
@@ -255,7 +261,8 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
         "beautifulsoup4>=4.7.1",
         # Coverage 7.4.0 added experimental support for Python 3.12 PEP669 which we use in Airflow
         "coverage>=7.4.0",
-        "pytest-asyncio>=0.23.3",
+        "jmespath>=0.7.0",
+        "pytest-asyncio>=0.23.6",
         "pytest-cov>=4.1.0",
         "pytest-custom-exit-code>=0.3.0",
         "pytest-icdiff>=0.9",
@@ -264,9 +271,7 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
         "pytest-rerunfailures>=13.0",
         "pytest-timeouts>=1.2.1",
         "pytest-xdist>=3.5.0",
-        # Temporary upper limmit to <8, not all dependencies at that moment ready to use 8.0
-        # Internal meta-task for track https://github.com/apache/airflow/issues/37156
-        "pytest>=7.4.4,<8.0",
+        "pytest>=8.2,<9",
         "requests_mock>=1.11.0",
         "time-machine>=2.13.0",
         "wheel>=0.42.0",
@@ -274,8 +279,6 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
     "devel": [
         "apache-airflow[celery]",
         "apache-airflow[cncf-kubernetes]",
-        "apache-airflow[common-io]",
-        "apache-airflow[common-sql]",
         "apache-airflow[devel-debuggers]",
         "apache-airflow[devel-devscripts]",
         "apache-airflow[devel-duckdb]",
@@ -283,11 +286,6 @@ DEVEL_EXTRAS: dict[str, list[str]] = {
         "apache-airflow[devel-sentry]",
         "apache-airflow[devel-static-checks]",
         "apache-airflow[devel-tests]",
-        "apache-airflow[fab]",
-        "apache-airflow[ftp]",
-        "apache-airflow[http]",
-        "apache-airflow[imap]",
-        "apache-airflow[sqlite]",
     ],
     "devel-all-dbs": [
         "apache-airflow[apache-cassandra]",
@@ -416,14 +414,12 @@ DEPENDENCIES = [
     # The 1.13.0 of alembic marked some migration code as SQLAlchemy 2+ only so we limit it to 1.13.1
     "alembic>=1.13.1, <2.0",
     "argcomplete>=1.10",
-    "asgiref",
+    "asgiref>=2.3.0",
     "attrs>=22.1.0",
     # Blinker use for signals in Flask, this is an optional dependency in Flask 2.2 and lower.
     # In Flask 2.3 it becomes a mandatory dependency, and flask signals are always available.
     "blinker>=1.6.2",
-    # Colorlog 6.x merges TTYColoredFormatter into ColoredFormatter, breaking backwards compatibility with 4.x
-    # Update CustomTTYColoredFormatter to remove
-    "colorlog>=4.0.2, <5.0",
+    "colorlog>=6.8.2",
     "configupdater>=3.1.1",
     # `airflow/www/extensions/init_views` imports `connexion.decorators.validation.RequestBodyValidator`
     # connexion v3 has refactored the entire module to middleware, see: /spec-first/connexion/issues/1525
@@ -431,25 +427,26 @@ DEPENDENCIES = [
     # The usage was added in #30596, seemingly only to override and improve the default error message.
     # Either revert that change or find another way, preferably without using connexion internals.
     # This limit can be removed after https://github.com/apache/airflow/issues/35234 is fixed
-    "connexion[flask]>=2.10.0,<3.0",
+    "connexion[flask]>=2.14.2,<3.0",
     "cron-descriptor>=1.2.24",
     "croniter>=2.0.2",
-    "cryptography>=39.0.0",
+    "cryptography>=41.0.0",
     "deprecated>=1.2.13",
     "dill>=0.2.2",
-    "flask-caching>=1.5.0",
+    "flask-caching>=2.0.0",
     # Flask-Session 0.6 add new arguments into the SqlAlchemySessionInterface constructor as well as
     # all parameters now are mandatory which make AirflowDatabaseSessionInterface incopatible with this version.
     "flask-session>=0.4.0,<0.6",
-    "flask-wtf>=0.15",
+    "flask-wtf>=1.1.0",
     # Flask 2.3 is scheduled to introduce a number of deprecation removals - some of them might be breaking
     # for our dependencies - notably `_app_ctx_stack` and `_request_ctx_stack` removals.
     # We should remove the limitation after 2.3 is released and our dependencies are updated to handle it
-    "flask>=2.2,<2.3",
+    "flask>=2.2.1,<2.3",
     "fsspec>=2023.10.0",
-    "google-re2>=1.0",
+    'google-re2>=1.0;python_version<"3.12"',
+    'google-re2>=1.1;python_version>="3.12"',
     "gunicorn>=20.1.0",
-    "httpx",
+    "httpx>=0.25.0",
     'importlib_metadata>=6.5;python_version<"3.12"',
     # Importib_resources 6.2.0-6.3.1 break pytest_rewrite
     # see https://github.com/python/importlib_resources/issues/299
@@ -457,7 +454,7 @@ DEPENDENCIES = [
     "itsdangerous>=2.0",
     "jinja2>=3.0.0",
     "jsonschema>=4.18.0",
-    "lazy-object-proxy",
+    "lazy-object-proxy>=1.2.0",
     "linkify-it-py>=2.0.0",
     "lockfile>=0.12.2",
     "markdown-it-py>=2.1.0",
@@ -466,24 +463,26 @@ DEPENDENCIES = [
     "mdit-py-plugins>=0.3.0",
     "methodtools>=0.4.7",
     "opentelemetry-api>=1.15.0",
-    "opentelemetry-exporter-otlp",
-    "packaging>=14.0",
+    "opentelemetry-exporter-otlp>=1.15.0",
+    "packaging>=23.0",
     "pathspec>=0.9.0",
-    "pendulum>=2.1.2,<4.0",
-    "pluggy>=1.0",
-    "psutil>=4.2.0",
+    'pendulum>=2.1.2,<4.0;python_version<"3.12"',
+    'pendulum>=3.0.0,<4.0;python_version>="3.12"',
+    "pluggy>=1.5.0",
+    "psutil>=5.8.0",
     "pygments>=2.0.1",
     "pyjwt>=2.0.0",
     "python-daemon>=3.0.0",
-    "python-dateutil>=2.3",
+    "python-dateutil>=2.7.0",
     "python-nvd3>=0.15.0",
     "python-slugify>=5.0",
     # Requests 3 if it will be released, will be heavily breaking.
     "requests>=2.27.0,<3",
+    "requests-toolbelt>=0.4.0",
     "rfc3339-validator>=0.1.4",
     "rich-argparse>=1.0.0",
     "rich>=12.4.4",
-    "setproctitle>=1.1.8",
+    "setproctitle>=1.3.3",
     # We use some deprecated features of sqlalchemy 2.0 and we should replace them before we can upgrade
     # See https://sqlalche.me/e/b8d9 for details of deprecated features
     # you can set environment variable SQLALCHEMY_WARN_20=1 to show all deprecation warnings.
@@ -491,12 +490,12 @@ DEPENDENCIES = [
     "sqlalchemy>=1.4.36,<2.0",
     "sqlalchemy-jsonfield>=1.0",
     "tabulate>=0.7.5",
-    "tenacity>=6.2.0,!=8.2.0",
+    "tenacity>=8.0.0,!=8.2.0",
     "termcolor>=1.1.0",
     # We should remove this dependency when Providers are limited to Airflow 2.7+
     # as we replaced the usage of unicodecsv with csv in Airflow 2.7
     # See https://github.com/apache/airflow/pull/31693
-    # We should also remove "licenses/LICENSE-unicodecsv.txt" file when we remove this dependency
+    # We should also remove "3rd-party-licenses/LICENSE-unicodecsv.txt" file when we remove this dependency
     "unicodecsv>=0.14.1",
     # The Universal Pathlib provides  Pathlib-like interface for FSSPEC
     "universal-pathlib>=0.2.2",
@@ -551,11 +550,35 @@ ALL_DYNAMIC_EXTRAS: list[str] = sorted(
 
 
 def get_provider_id(provider_spec: str) -> str:
-    # in case provider_spec is "<provider_id>=<version>"
-    return provider_spec.split(">=")[0]
+    """
+    Extract provider id from provider specification.
+
+    :param provider_spec: provider specification can be in the form of the "PROVIDER_ID" or
+           "apache-airflow-providers-PROVIDER", optionally followed by ">=VERSION".
+
+    :return: short provider_id with `.` instead of `-` in case of `apache` and other providers with
+             `-` in the name.
+    """
+    _provider_id = provider_spec.split(">=")[0]
+    if _provider_id.startswith("apache-airflow-providers-"):
+        _provider_id = _provider_id.replace("apache-airflow-providers-", "").replace("-", ".")
+    return _provider_id
 
 
 def get_provider_requirement(provider_spec: str) -> str:
+    """
+    Convert provider specification with provider_id to provider requirement.
+
+    The requirement can be used when constructing dependencies. It automatically adds pre-release specifier
+    in case we are building pre-release version of Airflow. This way we can handle the case when airflow
+    depends on specific version of the provider that has not yet been released - then we release the
+    pre-release version of provider to PyPI and airflow built in CI, or Airflow pre-release version will
+    automatically depend on that pre-release version of the provider.
+
+    :param provider_spec: provider specification can be in the form of the "PROVIDER_ID" optionally followed
+       by >=VERSION.
+    :return: requirement for the provider that can be used as dependency.
+    """
     if ">=" in provider_spec:
         # we cannot import `airflow` here directly as it would pull re2 and a number of airflow
         # dependencies so we need to read airflow version by matching a regexp
@@ -578,29 +601,48 @@ def get_provider_requirement(provider_spec: str) -> str:
         return f"apache-airflow-providers-{provider_spec.replace('.', '-')}"
 
 
-# if providers are ready, we can preinstall them
-PREINSTALLED_PROVIDERS = [
+# if providers are ready, we build provider requirements for them
+PREINSTALLED_PROVIDER_REQUIREMENTS = [
     get_provider_requirement(provider_spec)
     for provider_spec in PRE_INSTALLED_PROVIDERS
     if PROVIDER_DEPENDENCIES[get_provider_id(provider_spec)]["state"] == "ready"
 ]
 
-# if provider is in not-ready or pre-release, we need to install its dependencies
-# however we need to skip apache-airflow itself and potentially any providers that are
-PREINSTALLED_NOT_READY_DEPS = []
+# Here we keep all pre-installed provider dependencies, so that we can add them as requirements in
+# editable build to make sure that all dependencies are installed when we install Airflow in editable mode
+# We need to skip apache-airflow min-versions and flag (exit) when pre-installed provider has
+# dependency to another provider
+ALL_PREINSTALLED_PROVIDER_DEPS: list[str] = []
+
+# We very rarely - and only for the time when we plan to release a new preinstalled provider in next release
+# we have the preinstalled provider that is in non-ready state.
+# If provider is in not-ready state, we need to install its dependencies in editable mode as well as
+# when we are building the wheel in CI. In pre-release branch we should never have a non-ready provider
+# added, so this will only be used in main branch for CI builds.
+PREINSTALLED_NOT_READY_PROVIDER_DEPS: list[str] = []
+
 for provider_spec in PRE_INSTALLED_PROVIDERS:
     provider_id = get_provider_id(provider_spec)
-    if PROVIDER_DEPENDENCIES[provider_id]["state"] not in ["ready", "suspended", "removed"]:
-        for dependency in PROVIDER_DEPENDENCIES[provider_id]["deps"]:
-            if dependency.startswith("apache-airflow-providers"):
-                msg = (
-                    f"The provider {provider_id} is pre-installed and it has as dependency "
-                    f"to another provider {dependency}. This is not allowed. Pre-installed"
-                    f"providers should only have 'apache-airflow' and regular dependencies."
-                )
-                raise SystemExit(msg)
-            if not dependency.startswith("apache-airflow"):
-                PREINSTALLED_NOT_READY_DEPS.append(dependency)
+    for dependency in PROVIDER_DEPENDENCIES[provider_id]["deps"]:
+        if (
+            dependency.startswith("apache-airflow-providers")
+            and get_provider_id(dependency) not in PRE_INSTALLED_PROVIDERS
+        ):
+            msg = (
+                f"The provider {provider_id} is pre-installed and it has a dependency "
+                f"to another provider {dependency} which is not preinstalled. This is not allowed. "
+                f"Pre-installed providers should only have 'apache-airflow', other preinstalled providers"
+                f"and regular non-airflow dependencies."
+            )
+            raise SystemExit(msg)
+        if not dependency.startswith("apache-airflow"):
+            if PROVIDER_DEPENDENCIES[provider_id]["state"] not in ["suspended", "removed"]:
+                ALL_PREINSTALLED_PROVIDER_DEPS.append(dependency)
+                if PROVIDER_DEPENDENCIES[provider_id]["state"] in ["not-ready"]:
+                    PREINSTALLED_NOT_READY_PROVIDER_DEPS.append(dependency)
+
+ALL_PREINSTALLED_PROVIDER_DEPS = sorted(set(ALL_PREINSTALLED_PROVIDER_DEPS))
+PREINSTALLED_NOT_READY_PROVIDER_DEPS = sorted(set(PREINSTALLED_NOT_READY_PROVIDER_DEPS))
 
 
 class CustomBuild(BuilderInterface[BuilderConfig, PluginManager]):
@@ -609,7 +651,6 @@ class CustomBuild(BuilderInterface[BuilderConfig, PluginManager]):
     # Note that this name of the plugin MUST be `custom` - as long as we use it from custom
     # hatch_build.py file and not from external plugin. See note in the:
     # https://hatch.pypa.io/latest/plugins/build-hook/custom/#example
-    #
     PLUGIN_NAME = "custom"
 
     def clean(self, directory: str, versions: Iterable[str]) -> None:
@@ -690,16 +731,29 @@ GENERATED_DEPENDENCIES_START = "# START OF GENERATED DEPENDENCIES"
 GENERATED_DEPENDENCIES_END = "# END OF GENERATED DEPENDENCIES"
 
 
-def convert_to_extra_dependency(dependency: str) -> str:
+def convert_to_extra_dependency(provider_requirement: str) -> str:
+    """
+    Convert provider specification to extra dependency.
+
+    :param provider_requirement: requirement of the provider in the form of apache-airflow-provider-*,
+        optionally followed by >=VERSION.
+    :return: extra dependency in the form of apache-airflow[extra]
+    """
     # if there is version in dependency - remove it as we do not need it in extra specification
     # for editable installation
-    if ">=" in dependency:
-        dependency = dependency.split(">=")[0]
-    extra = dependency.replace("apache-airflow-providers-", "").replace("-", "_").replace(".", "_")
+    if ">=" in provider_requirement:
+        provider_requirement = provider_requirement.split(">=")[0]
+    extra = provider_requirement.replace("apache-airflow-providers-", "").replace("-", "_").replace(".", "_")
     return f"apache-airflow[{extra}]"
 
 
 def get_python_exclusion(excluded_python_versions: list[str]):
+    """
+    Produce the Python exclusion that should be used - converted from the list of python versions.
+
+    :param excluded_python_versions: list of python versions to exclude the dependency for.
+    :return: python version exclusion string that can be added to dependency in specification.
+    """
     exclusion = ""
     if excluded_python_versions:
         separator = ";"
@@ -710,6 +764,12 @@ def get_python_exclusion(excluded_python_versions: list[str]):
 
 
 def skip_for_editable_build(excluded_python_versions: list[str]) -> bool:
+    """
+    Whether the dependency should be skipped for editable build for current python version.
+
+    :param excluded_python_versions: list of excluded python versions.
+    :return: True if the dependency should be skipped for editable build for the current python version.
+    """
     current_python_version = f"{sys.version_info.major}.{sys.version_info.minor}"
     if current_python_version in excluded_python_versions:
         return True
@@ -717,7 +777,23 @@ def skip_for_editable_build(excluded_python_versions: list[str]) -> bool:
 
 
 class CustomBuildHook(BuildHookInterface[BuilderConfig]):
-    """Custom build hook for Airflow - remove devel extras and adds preinstalled providers."""
+    """
+    Custom build hook for Airflow.
+
+    Generates required and optional dependencies depends on the build `version`.
+
+    - standard: Generates all dependencies for the standard (.whl) package:
+       * devel and doc extras not included
+       * core extras and "production" bundle extras included
+       * provider optional dependencies resolve to "apache-airflow-providers-{provider}"
+       * pre-installed providers added as required dependencies
+
+    - editable: Generates all dependencies for the editable installation:
+       * devel and doc extras (including devel bundle extras are included)
+       * core extras and "production" bundles included
+       * provider optional dependencies resolve to provider dependencies including devel dependencies
+       * pre-installed providers not included - their dependencies included in devel extras
+    """
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         # Stores all dependencies that that any of the airflow extras (including devel) use
@@ -735,6 +811,9 @@ class CustomBuildHook(BuildHookInterface[BuilderConfig]):
         Initialize hook immediately before each build.
 
         Any modifications to the build data will be seen by the build target.
+
+        :param version: "standard" or "editable" build.
+        :param build_data: build data dictionary.
         """
         self._process_all_built_in_extras(version)
         self._process_all_provider_extras(version)
@@ -760,10 +839,10 @@ class CustomBuildHook(BuildHookInterface[BuilderConfig]):
 
         if version == "standard":
             # Inject preinstalled providers into the dependencies for standard packages
-            for provider in PREINSTALLED_PROVIDERS:
-                self._dependencies.append(provider)
-            for not_ready_provider_dependency in PREINSTALLED_NOT_READY_DEPS:
-                self._dependencies.append(not_ready_provider_dependency)
+            self._dependencies.extend(PREINSTALLED_PROVIDER_REQUIREMENTS)
+            self._dependencies.extend(PREINSTALLED_NOT_READY_PROVIDER_DEPS)
+        else:
+            self._dependencies.extend(ALL_PREINSTALLED_PROVIDER_DEPS)
 
         # with hatchling, we can modify dependencies dynamically by modifying the build_data
         build_data["dependencies"] = self._dependencies
@@ -773,16 +852,27 @@ class CustomBuildHook(BuildHookInterface[BuilderConfig]):
         # field in core.metadata until this is possible
         self.metadata.core._optional_dependencies = self.optional_dependencies
 
+        # Add entrypoints dynamically for all provider packages, in editable build
+        # else they will not be found by plugin manager
+        if version != "standard":
+            entry_points = self.metadata.core._entry_points or {}
+            plugins = entry_points.get("airflow.plugins") or {}
+            for provider in PROVIDER_DEPENDENCIES.values():
+                for plugin in provider["plugins"]:
+                    plugin_class: str = plugin["plugin-class"]
+                    plugins[plugin["name"]] = plugin_class[::-1].replace(".", ":", 1)[::-1]
+            entry_points["airflow.plugins"] = plugins
+            self.metadata.core._entry_points = entry_points
+
     def _add_devel_ci_dependencies(self, deps: list[str], python_exclusion: str) -> None:
         """
         Add devel_ci_dependencies.
 
         Adds all external dependencies which are not apache-airflow deps to the list of dependencies
-        that are going to be added to `devel-ci` extra.
+        that are going to be added to `devel-ci` extra. Optionally exclude dependencies for specific
+        python versions.
 
         :param deps: list of dependencies to add
-        :param version: "standard" or "editable" build.
-        :param excluded_python_versions: List of python versions to exclude
         :param python_exclusion: Python version exclusion string.
         """
         for dep in deps:
@@ -797,7 +887,6 @@ class CustomBuildHook(BuildHookInterface[BuilderConfig]):
         and providers for wheel builds.
 
         :param version: "standard" or "editable" build.
-        :return:
         """
         for dependency_id in PROVIDER_DEPENDENCIES.keys():
             if PROVIDER_DEPENDENCIES[dependency_id]["state"] != "ready":
@@ -840,7 +929,6 @@ class CustomBuildHook(BuildHookInterface[BuilderConfig]):
         used to produce "all" extra.
 
         :param version: "standard" or "editable" build.
-        :return:
         """
         for dict, _ in ALL_DYNAMIC_EXTRA_DICTS:
             for extra, deps in dict.items():
